﻿Option Explicit On

'  ヒュッケル分子軌道法計算のプログラム
'  Formで対話的に項目を指定し、永年行列を自動作成する版 

'　                         Written by Hiroshi Kihara

'  奇数電子系およびHOMOの二重縮退に対応しています。

Public Class Form1

    Const M = 20

    Dim compName As String
    Dim NA As Integer       '  原子数
    Dim NB As Integer        '  結合数
    Dim NP As Integer        '  π電子数
    Dim Charge As Integer
    Dim Cyclo As Boolean
    Dim G(M, M) As Double           ' 永年行列
    Dim V(M, M) As Double           ' 固有ベクトル
    Dim P(M, M) As Double           ' 結合次数
    Public printData As String


    Private Sub Button1_Click(sender As Object, e As EventArgs) Handles Button1.Click

        If NA < 2 Then
            MessageBox.Show("原子数は２以上を入力してください。")
            Exit Sub
        End If

        If Cyclo Then
            NB = NA
        Else
            NB = NA - 1
        End If

        Call SECDET()    ' 永年行列の組み立て

        Call DIAG()      'ヤコビ法による対角化

        Call INCRE()     ' 固有ベクトルを固有値の順に並べかえる

        Call PMAT()     ' 結合次数の計算

        Call POUT()     ' 結果の表示

    End Sub

    Sub SECDET()

        ' 永年行列の組み立て

        Dim i As Integer
        Dim j As Integer

        NP = NA - Charge

        '  結合している原子の番号の組 (i,j)
        For i = 1 To NA
            For j = 1 To NA
                G(i, j) = 0.0#
            Next j
        Next i

        '  結合している原子の番号の組 (i,j)
        For i = 1 To NA - 1
            G(i, i + 1) = 1.0#
            G(i + 1, i) = 1.0#
        Next i

        If Cyclo Then
            G(1, NA) = 1.0#
            G(NA, 1) = 1.0#
        End If

    End Sub

    Sub DIAG()

        'ヤコビ法による対角化

        Dim I As Integer
        Dim J As Integer
        Dim K As Integer
        Dim L As Integer
        Dim IT As Integer
        Dim D As Double
        Dim SM As Double
        Dim CS As Double
        Dim SN As Double
        Dim TA As Double
        Dim TB As Double
        Dim GJ As Double
        Dim GK As Double
        Dim GJL As Double
        Dim GKL As Double
        Dim VJ As Double
        Dim VK As Double

        ' 初期化

        Const DL = 0.000001

        For I = 1 To NA
            For J = 1 To NA
                V(I, J) = 0.0#
            Next J
        Next I

        For I = 1 To NA
            V(I, I) = 1.0#
        Next I

        D = 1.0#

        Do While D > DL

            D = Math.Abs(G(2, 1))

            For I = 2 To NA
                For J = 1 To I - 1
                    If Math.Abs(G(I, J)) > D Then D = Math.Abs(G(I, J))
                Next J
            Next I

            SM = 0.1 * D
            IT = IT + 1

            For J = 2 To NA

                For K = 1 To J - 1

                    If Math.Abs(G(J, K)) > SM Then

                        GJ = G(J, J)
                        GK = G(K, K)
                        TB = (GJ - GK) / (2 * G(K, J))

                        If TB <= 0 Then TA = TB - Math.Sqrt(1 + TB * TB) Else TA = TB + Math.Sqrt(1 + TB * TB)
                        CS = 1 / Math.Sqrt(1 + TA * TA)
                        SN = CS * TA

                        For L = 1 To NA

                            If L = K Then
                                G(K, K) = GK * CS * CS + G(J, K) * 2 * SN * CS + GJ * SN * SN
                                G(J, J) = GK + GJ - G(K, K)

                            ElseIf L <> J Then

                                GKL = G(K, L)
                                GJL = G(J, L)
                                G(K, L) = GKL * CS + GJL * SN
                                G(J, L) = GJL * CS - GKL * SN
                                G(L, K) = G(K, L)
                                G(L, J) = G(J, L)

                            End If

                        Next L

                        G(J, K) = G(J, K) * (CS * CS - SN * SN) + (GJ - GK) * SN * CS
                        G(K, J) = G(J, K)

                        For I = 1 To NA
                            VJ = V(I, J)
                            VK = V(I, K)
                            V(I, J) = VJ * CS - VK * SN
                            V(I, K) = VJ * SN + VK * CS
                        Next I

                    End If

                Next K

            Next J

        Loop

    End Sub

    Sub INCRE()

        ' 固有ベクトルを固有値の順に並べかえる

        Dim i As Integer
        Dim j As Integer
        Dim imax As Integer
        Dim Emax As Double
        Dim T As Double

        For i = 1 To NA
            Emax = G(i, i)
            imax = i

            For j = i To NA
                If G(j, j) > Emax Then
                    imax = j
                    Emax = G(j, j)
                End If
            Next j

            If i < imax Then

                T = G(i, i)
                G(i, i) = G(imax, imax)
                G(imax, imax) = T

                For j = 1 To NA
                    T = V(j, i)
                    V(j, i) = V(j, imax)
                    V(j, imax) = T
                Next j

            End If

        Next i
    End Sub
    Sub PMAT()

        ' 結合次数の計算

        Dim i As Integer
        Dim j As Integer
        Dim k As Integer
        Dim NOCC As Integer
        Dim Odd As Boolean
        Dim Degenerate As Boolean
        Const EPS = 0.000001#

        NOCC = (NP + 1) \ 2
        If NOCC * 2 > NP Then Odd = True

        ' HOMOと一つ上のMOのエネルギーを比較
        If G(NOCC, NOCC) - G(NOCC + 1, NOCC + 1) < EPS Then Degenerate = True

        For i = 1 To NA

            For j = 1 To i

                P(i, j) = 0.0#

                For k = 1 To NOCC - 1

                    P(i, j) = P(i, j) + 2.0# * V(i, k) * V(j, k)

                Next k

                If Degenerate Then
                    If Odd Then
                        P(i, j) = P(i, j) + 0.5 * (V(i, NOCC) * V(j, NOCC) + V(i, NOCC + 1) * V(j, NOCC + 1))
                    Else
                        P(i, j) = P(i, j) + V(i, NOCC) * V(j, NOCC) + V(i, NOCC + 1) * V(j, NOCC + 1)
                    End If
                Else
                    If Odd Then
                        P(i, j) = P(i, j) + V(i, NOCC) * V(j, NOCC)
                    Else
                        P(i, j) = P(i, j) + 2.0# * V(i, NOCC) * V(j, NOCC)
                    End If

                End If

            Next j

        Next i

    End Sub
    Sub POUT()

        Dim i As Integer
        Dim j As Integer
        Dim k As Integer
        Dim m As Integer
        Const nItem = 8

        Dim f2 As New Form2

        printData = "化合物名: " & compName & vbCrLf
        printData &= "原子数: " & Str(NA) & vbCrLf
        printData &= "結合数: " & Str(NB) & vbCrLf
        printData &= "π電子数: " & Str(NP) & vbCrLf & vbCrLf

        ' 固有値と固有ベクトルの表示
        printData &= "-- Eigenvalues and Eigenvectors --" & vbCrLf & vbCrLf

        k = 1     ' 開始列
        m = k + nItem - 1

        Do While k <= NA

            If m > NA Then m = NA

            '            printData &= Space(10)

            For i = k To m
                printData &= Space(9) & i.ToString("D2")
            Next i

            printData &= vbCrLf

        ' 固有値（エネルギー）
            printData &= Space(2)

            For i = k To m
                printData &= Space(3) & G(i, i).ToString(" 0.####0;-0.####0")
            Next i

 
        printData &= vbCrLf & vbCrLf

        ' 固有ベクトル（波動関数)
            For i = 1 To NA
                printData &= i.ToString("D2")
                For j = k To m
                    printData &= Space(3) & V(i, j).ToString(" 0.####0;-0.####0")
                Next j
                printData &= vbCrLf
            Next i

            k = m + 1
            m = k + nItem - 1
            printData &= vbCrLf

        Loop

        ' 結合次数の表示

        printData &= vbCrLf & "-- Bond Order Matrix --" & vbCrLf & vbCrLf
        k = 1
        m = k + nItem - 1

        Do While k <= NA

            If m > NA Then m = NA

            '            printData &= Space(10)
            For i = k To m
                printData &= Space(9) & i.ToString("D2")
            Next i
            printData &= vbCrLf

            ' 結合次数の表示
            For i = 1 To NA
                printData &= i.ToString("D2")
                For j = k To m
                    printData &= Space(3) & P(i, j).ToString(" 0.####0;-0.####0")
                Next j
                printData &= vbCrLf
            Next i

            k = m + 1
            m = k + nItem - 1
            printData &= vbCrLf

        Loop

        f2.Show(Me)

        ' テキストボックスに表示する
        f2.TextBox1.Text = printData

    End Sub

    Private Sub TextBox1_TextChanged(sender As Object, e As EventArgs) Handles TextBox1.TextChanged
        compName = TextBox1.Text       ' 化合物名
        RadioButton2.Enabled = True
    End Sub

    Private Sub TextBox2_TextChanged(sender As Object, e As EventArgs) Handles TextBox2.TextChanged
        RadioButton2.Enabled = True
        NA = CInt(TextBox2.Text)

        ' 炭素数が３以下の時は、環状を指定しても無効にする
        If NA < 3 Then
            RadioButton2.Enabled = False
        End If

        If NA > 20 Then
            MessageBox.Show("原子数の上限は20です。")
            Exit Sub
        End If

    End Sub

    Private Sub RadioButton1_CheckedChanged(sender As Object, e As EventArgs) Handles RadioButton1.CheckedChanged
        '  環状か否か
        If RadioButton1.Checked Then Cyclo = False
 
    End Sub

    Private Sub RadioButton2_CheckedChanged(sender As Object, e As EventArgs) Handles RadioButton2.CheckedChanged
        '  環状か否か
        If RadioButton2.Checked Then Cyclo = True
        
    End Sub

    Private Sub RadioButton4_CheckedChanged(sender As Object, e As EventArgs) Handles RadioButton4.CheckedChanged
        If RadioButton4.Checked Then Charge = 1
    End Sub

    Private Sub RadioButton3_CheckedChanged(sender As Object, e As EventArgs) Handles RadioButton3.CheckedChanged
        If RadioButton3.Checked Then Charge = 0
    End Sub

    Private Sub RadioButton5_CheckedChanged(sender As Object, e As EventArgs) Handles RadioButton5.CheckedChanged
        If RadioButton5.Checked Then Charge = -1
    End Sub
End Class
